/* SIM1.PRG 

This Gauss code executes the first simulation reported in "Forecast Model Averaging" by
Bruce E. Hansen, http://www.ssc.wisc.edu/~bhansen/

*/


rep=20000;
cn=17; rs=seqa(.1,.05,cn);	@ for graphs @
@cn=9; rs=seqa(.1,.1,cn);@	@ for drafts @
cs=sqrt(rs./(1-rs));
n=200;
k=12;
nm=11;
kk=7;				@ number of models for estimation, kk=k+1 or kk=7 @

library pgraph;
pqgwin many;

xg=seqa(1,1,k);
xgk=xg/(k+1);

let mas[8,2] = 
-.5 0 
-.5 1
-.3 .2 
-.3 1
-.1 .2
-.1 .8
.1 .5
.3 .6;
mn=rows(mas);

gams=zeros(k,mn);

for mai (1,mn,1);
ma=mas[mai,.];

gam=(xgk.^ma[1]).*((1-xgk).^ma[2]);
gam=gam./sumc(gam.^2);
gams[.,mai]=gam;

mse=zeros(cn,nm);

for i (1,rep,1);
x=recserar(rndn(n+1,k),rndn(1,k),.5*ones(1,k))*sqrt(3/4);
mu=x[1:n,.]*gam;
muf=x[n+1,.]*gam;
e=rndn(n,1);
xf=1|(x[n+1,1:k]');
x=ones(n,1)~x[1:n,1:k];

store=zeros(cn,nm);

for ci (1,cn,1);
c=cs[ci];
yf=muf*c;
y=mu*c+e;

fhat=zeros(n,kk);
t1=round(n/2);
sxy=cumsumc(x.*y);
xxt=moment(x[1:t1-2,.],0);
for t (t1,n,1);
  xxt=xxt+moment(x[t-1,.],0);
  for j (1,kk,1);
    bj=(sxy[t-1,1:j]')/xxt[1:j,1:j];
    fhat[t,j]=x[t,1:j]*bj;
  endfor;
endfor;
xx=xxt+moment(x[n,.],0);
sxy=sxy[n,.]';

fy=zeros(kk,1);
ehat=zeros(n,kk);
for j (1,kk,1);
  xj=x[.,1:j];
  bj=sxy[1:j]/xx[1:j,1:j];
  fy[j]=xf[1:j]'bj;
  ehat[.,j]=y-xj*bj;
endfor;

a_mma=ehat'ehat;
see=diag(a_mma);
s=seqa(1,1,kk);
sig=see[kk]/(n-kk);
aic=n*ln(see/n)+s*2;
bic=n*ln(see/n)+s*ln(n);
b_mma=-s*sig;
sf=(y-fhat).^2;
sf=sumc(sf[t1:n,.]);
a_gr=moment(fhat,0);
b_gr=fhat'y;

f1=median(fy);			@ Median @
f2=meanc(fy);			@ Mean (Equal Weights) @
f3=fy[minindc(bic)];		@ BIC @
f4=fy[minindc(aic)];		@ AIC @
f5=fy'eweight(bic);		@ WBIC @
f6=fy'eweight(aic);		@ WAIC @
f7=fy'((1./sf)/sumc(1./sf));	@ BG (Bates-Granger) @
f8=fy[minindc(sf)]; 		@ PLS (Predictive Least Squares) @
f9=fy'weight(a_gr,b_gr,0,0);	@ GR (Granger-Ramanathan) @
f10=fy'weight(a_gr,b_gr,1,1);	@ CGR (Constrained Granger-Ramanathan) @
f11=fy'weight(a_mma,b_mma,1,1);	@ MMA (Mallows Model Averaging) @

fall=f1~f2~f3~f4~f5~f6~f7~f8~f9~f10~f11;

store[ci,.]=((yf-fall).^2)*n;

endfor; @ ci @

mse=mse+store;
endfor; @ i @
mse=mse/rep;
hm="mse" $+ ftocv(mai,1,0);
ret=varput(mse,hm);

endfor; @ m @

regret=zeros(8,nm);
for m (1,8,1);
  hm="mse" $+ ftocv(m,1,0);
  mse=varget(hm);
  m1=minc(mse');
  regret[m,.]=maxc(mse-m1)';  
endfor;

let xnames= "Median" "Mean" "BIC" "AIC" "WBIC" "WAIC" "BG" "PLS" "GR" "CGR" "MMA";

output file = sim1.alt.out reset;
"Maxmimum Regret of all procedures";"";
pf=printfmt(xnames~maxc(regret),0~1);
output off;


graphset;
_pdate="";
_plwidth=4;
let _pmcolor = 0 0 0 0 0 0 0 0 15;
fonts("complex simgrma");
_pcolor=1;
_paxht=.3;
_pnumht=.25;
_protate=1;
_plegstr="WBIC\000Bates-Granger\000MMA";
_ptek="fig1.tkf";
begwind;
window(4,2,1);
makewind(9,6.855,0,0,1);
for mai (1,4,1);
setwind(mai*2-1);
  _plegctl=0;
  _pltype= 6;
  gam=gams[.,mai];
  scale(xg,gam);
  xtics(1,k,1,1);
  xlabel("\202g\201]k[");
  ylabel("Regression Coefficients");
  xy(xg,gam);
setwind(mai*2);
  let _pltype= 3 5 6 4;
  _plegctl=1;
  hm="mse" $+ ftocv(mai,1,0);
  mse=varget(hm);
  ytics(0,20,5,5,);
  xtics(.1,.9,.1,2);
  xlabel("R[2]");
  ylabel("MSFE");
  xy(rs,mse[.,5 7 11]);
endfor;
setwind(9);
ylabel("");
xlabel("");
_paxes=0;
_pmsgstr = "" $+ "Figure 1: Regression Model, Cases 1-4";
let _pmsgctl[1,7] = 1 8.8 .15 0 2 1 0;
draw;
endwind;

graphset;
_pdate="";
_plwidth=4;
let _pmcolor = 0 0 0 0 0 0 0 0 15;
fonts("complex simgrma");
_pcolor=1;
_paxht=.3;
_pnumht=.25;
_protate=1;
_plegstr="WBIC\000Bates-Granger\000MMA";
_ptek="fig2.tkf";
begwind;
window(4,2,1);
makewind(9,6.855,0,0,1);
for mi (1,4,1);
mai=4+mi;
setwind(mi*2-1);
  _plegctl=0;
  _pltype= 6;
  gam=gams[.,mai];
  scale(xg,gam);
  xtics(1,k,1,1);
  xlabel("\202g\201]k[");
  ylabel("Regression Coefficients");
  xy(xg,gam);
setwind(mi*2);
  let _pltype= 3 5 6 4;
  _plegctl=1;
  hm="mse" $+ ftocv(mai,1,0);
  mse=varget(hm);
  ytics(0,30,5,5);
  xtics(.1,.9,.1,2);
  xlabel("R[2]");
  ylabel("MSFE");
  xy(rs,mse[.,5 7 11]);
endfor;
setwind(9);
ylabel("");
xlabel("");
_paxes=0;
_pmsgstr = "" $+ "Figure 2: Regression Model, Cases 5-8";
let _pmsgctl[1,7] = 1 8.8 .15 0 2 1 0;
draw;
endwind;


graphset;
_ptek="figA.tkf";
_pdate="";
_plwidth=4;
let _pmcolor = 0 0 0 0 0 0 0 0 15;
fonts("complex simgrma");
_pcolor=1;
_paxht=.3;
_pnumht=.25;
_protate=1;
xtics(.1,.9,.1,2);
_plegctl=1;
let _pltype= 3 6;
_plegstr="BIC\000wBIC";
begwind;
window(4,2,1);
makewind(9,6.855,0,0,1);
for mai (1,8,1);
setwind(mai);
  hm="mse" $+ ftocv(mai,1,0);
  mse=varget(hm);
  ylabel("MSFE");
  xy(rs,mse[.,3 5]);
endfor;
setwind(9);
ylabel("");
xlabel("");
_paxes=0;
_pmsgstr = "" $+ "Regression Model: BIC, WBIC";
let _pmsgctl[1,7] = 1 8.8 .15 0 2 1 0;
draw;
endwind;

graphset;
_ptek="figB.tkf";
_pdate="";
_plwidth=4;
let _pmcolor = 0 0 0 0 0 0 0 0 15;
fonts("complex simgrma");
_pcolor=1;
_paxht=.3;
_pnumht=.25;
_protate=1;
xtics(.1,.9,.1,2);
_plegctl=1;
let _pltype= 3 1 6;
_plegstr="AIC\000wAIC\000MMA";
begwind;
window(4,2,1);
makewind(9,6.855,0,0,1);
for mai (1,8,1);
setwind(mai);
  hm="mse" $+ ftocv(mai,1,0);
  mse=varget(hm);
  ylabel("MSFE");
  xy(rs,mse[.,4 6 11]);
endfor;
setwind(9);
ylabel("");
xlabel("");
_paxes=0;
_pmsgstr = "" $+ "Regression Model: AIC, WAIC, MMA";
let _pmsgctl[1,7] = 1 8.8 .15 0 2 1 0;
draw;
endwind;

graphset;
_ptek="figC.tkf";
_pdate="";
_plwidth=4;
let _pmcolor = 0 0 0 0 0 0 0 0 15;
fonts("complex simgrma");
_pcolor=1;
_paxht=.3;
_pnumht=.25;
_protate=1;
xtics(.1,.9,.1,2);
_plegctl=1;
begwind;
window(4,2,1);
makewind(9,6.855,0,0,1);
for mai (1,8,1);
setwind(mai);
  hm="mse" $+ ftocv(mai,1,0);
  mse=varget(hm);
  ylabel("MSFE");
  let _pltype= 2 3 6;
  _plegstr="Median\000Mean\000Bates-Granger";
  xy(rs,mse[.,1 2 7]);
endfor;
setwind(9);
ylabel("");
xlabel("");
_paxes=0;
_pmsgstr = "" $+ "Regression Model: BG, Mean, Median";
let _pmsgctl[1,7] = 1 8.8 .15 0 2 1 0;
draw;
endwind;

graphset;
_ptek="figD.tkf";
_pdate="";
_plwidth=4;
let _pmcolor = 0 0 0 0 0 0 0 0 15;
fonts("complex simgrma");
_pcolor=1;
_paxht=.3;
_pnumht=.25;
_protate=1;
xtics(.1,.9,.1,2);
_plegctl=1;
let _pltype= 1 2 3 6;
_plegstr="PLS\000GR\000CGR\000MMA";
begwind;
window(4,2,1);
makewind(9,6.855,0,0,1);
for mai (1,8,1);
setwind(mai);
  hm="mse" $+ ftocv(mai,1,0);
  mse=varget(hm);
  ylabel("MSFE");
  xy(rs,mse[.,8 9 10 11]);
endfor;
setwind(9);
ylabel("");
xlabel("");
_paxes=0;
_pmsgstr = "" $+ "Regression Model: PLS, GR, CGR, MMA";
let _pmsgctl[1,7] = 1 8.8 .15 0 2 1 0;
draw;
endwind;


/****************************************************************/

proc eweight(w);
local ew;
  ew=exp(-w/2);
  ew=ew./sumc(ew);
retp(ew);
endp;

proc (2) = ma_sim(n,a);
local q,e;
  q=rows(a);
  e=rndn(n+q,1)|0;
  for i (1,q,1);
    e=e[2:rows(e)]+e[1:rows(e)-1]*a[i];
  endfor;
retp(e[1:n],e[n+1]);
endp;


proc (2) = ma_sim2(n,a,m);
local q,e;
  e=rndn(n+m,1)|0;
  e=e[1+m:rows(e)]+e[1:rows(e)-m]*a;
retp(e[1:n],e[n+1]);
endp;

/* Proc weight(A,B,S,C)

Minimizes the quadratic function 

    W'AW - 2*B'W

Subject to the following possible constraints:

If S=1, then sum(w)=1. 
If C=1, then 0<=w<=1.

*/

proc weight(a,b,s,c);
local m,r,ai,w,u1,u2,u3,u4,ret;
  m=rows(a);
  r=ones(m,1);
  if c==1;
    if s==1;
      {w,u1,u2,u3,u4,ret}=QProg(r/m,a,b,r',1,0,0,0~1);
    else;
      {w,u1,u2,u3,u4,ret}=QProg(r/m,a,b,0,0,0,0,0~1);
    endif;
  else;
    if s==1;
      ai=invpd(a);
      w=ai*(b+r*(1-r'ai*b)/(r'ai*r));
    else;
      w=b/a;
    endif;
  endif;
retp(w);
endp;