/* SIM2.PRG 
This Gauss code executes the second simulation reported in "Forecast Model Averaging" by
Bruce E. Hansen, http://www.ssc.wisc.edu/~bhansen/
*/

rep=20000;
nb=16; bs=seqa(.6,.02,nb);	@ final @
@nb=7; bs=seqa(.6,.05,nb);@	@ drafts @
n = 200;
k=12;
kk=k+1;
kt=80;
nm=11;

let as = 0 .25 .5 .75;
na=rows(as);

library pgraph;
pqgwin many;

xg=seqa(0,1,kt);

for m (1,na,1);
a=as[m];

mse=zeros(nb,nm);

for i (1,rep,1);

store=zeros(nb,nm);

for ci (1,nb,1);
b=bs[ci];
gam=((1+xg).^a).*(b.^xg);

{y,yf}=ma_sim2(n+k,gam);

x=ones(n+1,1);
for j (1,k,1);
  x=x~y[1+k-j:n+k+1-j];
endfor;
xf=x[n+1,.]';
x=x[1:n,.];
y=y[1+k:n+k];

fhat=zeros(n,kk);
t1=round(n/2);
sxy=cumsumc(x.*y);
xxt=moment(x[1:t1-2,.],0);
for t (t1,n,1);
  xxt=xxt+moment(x[t-1,.],0);
  for j (1,k+1,1);
    bj=(sxy[t-1,1:j]')/xxt[1:j,1:j];
    fhat[t,j]=x[t,1:j]*bj;
  endfor;
endfor;
xx=xxt+moment(x[n,.],0);
sxy=sxy[n,.]';

fy=zeros(kk,1);
ehat=zeros(n,kk);
for j (1,kk,1);
  xj=x[.,1:j];
  bj=sxy[1:j]/xx[1:j,1:j];
  fy[j]=xf[1:j]'bj;
  ehat[.,j]=y-xj*bj;
endfor;

a_mma=moment(ehat,0);
see=diag(a_mma);
s=seqa(1,1,kk);
sig=see[kk]/(n-kk);
aic=n*ln(see/n)+s*2;
bic=n*ln(see/n)+s*ln(n);
b_mma=-s*sig;
sf=(y-fhat).^2;
sf=sumc(sf[t1:n,.]);
a_gr=moment(fhat,0);
b_gr=fhat'y;

f1=median(fy);			@ Median @
f2=meanc(fy);			@ Mean (Equal Weights) @
f3=fy[minindc(bic)];		@ BIC @
f4=fy[minindc(aic)];		@ AIC @
f5=fy'eweight(bic);		@ WBIC @
f6=fy'eweight(aic);		@ WAIC @
f7=fy'((1./sf)/sumc(1./sf));	@ BG (Bates-Granger) @
f8=fy[minindc(sf)]; 		@ PLS (Predictive Least Squares) @
f9=fy'weight(a_gr,b_gr,0,0);	@ GR (Granger-Ramanathan) @
f10=fy'weight(a_gr,b_gr,1,1);	@ CGR (Constrained Granger-Ramanathan) @
f11=fy'weight(a_mma,b_mma,1,1);	@ MMA (Mallows Model Averaging) @

fall=f1~f2~f3~f4~f5~f6~f7~f8~f9~f10~f11;
store[ci,.]=((yf-fall).^2)*n;

endfor; @ ci @

mse=mse+store;
endfor; @ i @

mse=mse/rep;
hm="mse" $+ ftocv(m,1,0);
ret=varput(mse,hm);

endfor; @ m @

let bb[1,4]= .9 .8 .7 .6;
_pb="";
for bi (1,cols(bb),1);
  _pb=_pb $+ "\202b\201=" $+ ftocv(bb[bi],1,1);
  if bi<cols(bb); _pb=_pb $+ "\000"; endif;
endfor;

regret=zeros(4,nm);
for m (1,4,1);
  hm="mse" $+ ftocv(m,1,0);
  mse=varget(hm);
  m1=minc(mse');
  regret[m,.]=maxc(mse-m1)';  
endfor;

let xnames= "Median" "Mean" "BIC" "AIC" "WBIC" "WAIC" "BG" "PLS" "GR" "CGR" "MMA";

output file = sim2.out reset;
"Maxmimum Regret of all procedures";"";
pf=printfmt(xnames~maxc(regret),0~1);
output off;

graphset;
_pdate="";
_plwidth=4;
let _pmcolor = 0 0 0 0 0 0 0 0 15;
fonts("complex simgrma");
_pcolor=1;
_paxht=.3;
_pnumht=.25;
_protate=1;
_ptek="fig3.tkf";
_ptitlht=.4;
_plegctl=1;
_x = "" $+ "\202Q\201]k[=(1+k)\202[a]b\201[k]";
begwind;
window(4,2,1);
makewind(9,6.855,0,0,1);
for m (1,4,1);
setwind(m*2-1); 
  ylabel("MA Coefficients");
  let _pltype= 6 1 2 3;
  xtics(0,24,4,4);
  xlabel(_x);
  a=as[m];
  theta=((1+xg).^a).*(bb.^xg);
  _plegstr=_pb;
  _pmsgstr="" $+ "\202a\201=" $+ ftocv(a,1,2);
  let _pmsgctl[1,7] = 5 6 .4 0 2 1 0;
  xy(xg,theta);
setwind(m*2);
  ylabel("MSFE");
  _plegstr="WBIC\000Bates-Granger\000MMA";
  let _pltype= 3 5 6;
  hm="mse" $+ ftocv(m,1,0);
  mse=varget(hm);
  xtics(.6,.9,.05,5);
  xlabel("\202b\201");
  _pmsgstr = "";
  xy(bs,mse[.,5 7 11]);
endfor;
nextwind;
ylabel("");
xlabel("");
_paxes=0;
_pmsgstr = "" $+ "Figure 3: Moving Average Model";
let _pmsgctl[1,7] = 1.5 8.8 .15 0 2 1 0;
draw;
endwind;


graphset;
_ptek="figE.tkf";
_pdate="";
_plwidth=4;
let _pmcolor = 0 0 0 0 0 0 0 0 15;
fonts("complex simgrma");
_pcolor=1;
_paxht=.3;
_pnumht=.25;
_protate=1;
xtics(.6,.9,.05,5);
_plegctl=1;
begwind;
window(4,2,1);
makewind(9,6.855,0,0,1);
for m (1,4,1);
setwind(1+(m-1)*2);
  let _pltype= 3 6;
  _plegstr="BIC\000wBIC";
  hm="mse" $+ ftocv(m,1,0);
  mse=varget(hm);
  ylabel("MSFE");
  xy(bs,mse[.,3 5]);
endfor;
for m (1,4,1);
setwind(m*2);
  let _pltype= 3 1 6;
  _plegstr="AIC\000wAIC\000MMA";
  hm="mse" $+ ftocv(m,1,0);
  mse=varget(hm);
  ylabel("MSFE");
  xy(bs,mse[.,4 6 11]);
endfor;
setwind(9);
ylabel("");
xlabel("");
_paxes=0;
_pmsgstr = "" $+ "MA Model: BIC/WBIC, AIC/WAIC/MMA";
let _pmsgctl[1,7] = 1 8.8 .15 0 2 1 0;
draw;
endwind;

graphset;
_ptek="figF.tkf";
_pdate="";
_plwidth=4;
let _pmcolor = 0 0 0 0 0 0 0 0 15;
fonts("complex simgrma");
_pcolor=1;
_paxht=.3;
_pnumht=.25;
_protate=1;
xtics(.6,.9,.05,5);
_plegctl=1;
begwind;
window(4,2,1);
makewind(9,6.855,0,0,1);
for m (1,4,1);
setwind(1+(m-1)*2);
  hm="mse" $+ ftocv(m,1,0);
  mse=varget(hm);
  ylabel("MSFE");
  let _pltype= 2 3 6;
  _plegstr="Median\000Mean\000Bates-Granger";
  xy(bs,mse[.,1 2 7]);
endfor;
for m (1,4,1);
setwind(m*2);
  hm="mse" $+ ftocv(m,1,0);
  mse=varget(hm);
  ylabel("MSFE");
  let _pltype= 1 2 3 6;
  _plegstr="PLS\000GR\000CGR\000MMA";
  xy(bs,mse[.,8 9  10 11]);
endfor;
setwind(9);
ylabel("");
xlabel("");
_paxes=0;
_pmsgstr = "" $+ "MA Model: Mean/Median/BG, PLS/GR/CGR/MMA";
let _pmsgctl[1,7] = 1 8.8 .15 0 2 1 0;
draw;
endwind;


/****************************************************************/

proc eweight(w);
local ew;
  ew=exp(-w/2);
  ew=ew./sumc(ew);
retp(ew);
endp;

proc (2) = ar_sim(n,a);
local q,y;
  q=rows(a);
  y=rndn(n+q,1)|0;
  for i (1,q,1);
    y=recserar(y,y[1],a[i]);
  endfor;
retp(y[q+1:n+q],y[n+q+1]);
endp;


proc (2) = ma_sim(n,a);
local q,e;
  q=rows(a);
  e=rndn(n+q,1)|0;
  for i (1,q,1);
    e=e[2:rows(e)]+e[1:rows(e)-1]*a[i];
  endfor;
retp(e[1:n],e[n+1]);
endp;

proc (2) = ma_sim2(n,a);
local q,e,y;
  q=rows(a);
  e=rndn(n+q,1)|0;
  y=e[1+q:n+q+1];
  for i (1,q,1);
    y=y+e[1+q-i:n+q+1-i]*a[i];
  endfor;
retp(y[1:n],y[n+1]);
endp;

proc (2) = arma_sim(n,a,c);
local q,e,p,y;
  p=rows(a);
  q=rows(c);
  e=rndn(n+p+q,1)|0;
  for i (1,q,1);
    e=e[2:rows(e)]+e[1:rows(e)-1]*c[i];
  endfor;
  y=recserar(e,e[1:p],a);
retp(y[1+p:n+p],y[n+p+1]);
endp;

/* Proc weight(A,B,S,C)

Minimizes the quadratic function 

    W'AW - 2*B'W

Subject to the following possible constraints:

If S=1, then sum(w)=1. 
If C=1, then 0<=w<=1.

*/

proc weight(a,b,s,c);
local m,r,ai,w,u1,u2,u3,u4,ret;
  m=rows(a);
  r=ones(m,1);
  if c==1;
    if s==1;
      {w,u1,u2,u3,u4,ret}=QProg(r/m,a,b,r',1,0,0,0~1);
    else;
      {w,u1,u2,u3,u4,ret}=QProg(r/m,a,b,0,0,0,0,0~1);
    endif;
  else;
    if s==1;
      ai=invpd(a);
      w=ai*(b+r*(1-r'ai*b)/(r'ai*r));
    else;
      w=b/a;
    endif;
  endif;
retp(w);
endp;